
/***************************************************
* By CCSC compiler
*  File Name: MK335program_1716.c
 * rev 1.0
*  Description: analysis sound from microphone and turn on relay
*  RA0 in MSGEQ7 out (AN0)
*  RA1 out opamp out
*  RA2 in ADC, mic level (AN2)
*  RA3 in ADC, potentiometer (AN3)
*  RA4 in opamp in+, offset
*  RA5 in opemp in-, microphone
*  RA6 out MSGEQ7 strobe
*  RA7 out MSGEQ7 reset
*  RB0 out 8x8LED col1
*  RB1 out 8x8LED col2
*  RB2 out 8x8LED col3
*  RB3 out 8x8LED col4
*  RB4 out 8x8LED col5
*  RB5 out 8x8LED col6
*  RB6 out 8x8LED col7
*  RB7 out 8x8LED col8
*  RC0 out 8x8LED row code0
*  RC1 out 8x8LED row code1
*  RC2 out 8x8LED row code2
*  RC3 in SW SET FREQ
*  RC4 in na
*  RC5 in na
*  RC6 in na
*  RC7 out relay, active h
*  RE3 in na
*  COPYRIGHT 2019 MYCOMKITS.COM, owned by CNET LIMITED
* $B$3$N%W%m%0%i%`$NCx:n8"$O!"@=:n<T!V%^%$%3%s%-%C%H%I%C%H%3%`1?1D!!M-8B2q<R%/%M%C%H!W$K5"B0$7$^$9!#(J
* $B2f!9$OCx:n8"$rJ|4~$7$F$$$^$;$s$,!"Ev%W%m%0%i%`$r;H$C$?3X=,$NCf$G%W%m%0%i%`$r<+M3$KJQ99$7$F$*;H$$$/$@$5$$!#(J
* The copyright of this program belongs to the producer "MYCOMKITS.COM management company CNET LIMITED". 
* We do not abandon copyright, but you can use and change the program in learning using this program. 
***************************************************/
//
#include <16F1716.h>
#DEVICE ADC=10
#fuses	INTRC_IO, NOWDT, PUT, NOPROTECT, NOCLKOUT
#fuses	NOWRT, NOBROWNOUT, NODEBUG, NOLVP
#use	delay(CLOCK = 4000000)
//
#use   fast_io(a)  //a7 to 0
#use   fast_io(b)  //b7 to 0
#use   fast_io(c)  //C7 to 0
//
#define Amode 0x3F         //port A all input, except A6,7
#define Bmode 0x00         //port B all output
#define Cmode 0x08         //port C all output, except C3
#define AmodePU   0x00
#define BmodePU   0x00
#define CmodePU   0x08	// C3 pullup
#define MSGEQ7_RESET_ON output_high(PIN_A7)
#define MSGEQ7_RESET_OFF output_low(PIN_A7)
#define MSGEQ7_STROBE_ON output_high(PIN_A6)
#define MSGEQ7_STROBE_OFF output_low(PIN_A6)
#define LED_COL1_ON output_high(PIN_B0)
#define LED_COL1_OFF output_low(PIN_B0)
#define LED_COL2_ON output_high(PIN_B1)
#define LED_COL2_OFF output_low(PIN_B1)
#define LED_COL3_ON output_high(PIN_B2)
#define LED_COL3_OFF output_low(PIN_B2)
#define LED_COL4_ON output_high(PIN_B3)
#define LED_COL4_OFF output_low(PIN_B3)
#define LED_COL5_ON output_high(PIN_B4)
#define LED_COL5_OFF output_low(PIN_B4)
#define LED_COL6_ON output_high(PIN_B5)
#define LED_COL6_OFF output_low(PIN_B5)
#define LED_COL7_ON output_high(PIN_B6)
#define LED_COL7_OFF output_low(PIN_B6)
#define LED_COL8_ON output_high(PIN_B7)
#define LED_COL8_OFF output_low(PIN_B7)
#define LED_ROW_C0_ON output_high(PIN_C0)
#define LED_ROW_C0_OFF output_low(PIN_C0)
#define LED_ROW_C1_ON output_high(PIN_C1)
#define LED_ROW_C1_OFF output_low(PIN_C1)
#define LED_ROW_C2_ON output_high(PIN_C2)
#define LED_ROW_C2_OFF output_low(PIN_C2)
#define RELAY_ON output_high(PIN_C7)
#define RELAY_OFF output_low(PIN_C7)
#define LEVEL_8	0x0332	// input - 500mv(OFFSET) > 4.0V
#define LEVEL_7	0x0243	// LEVEL_8 - 3dB 2.83V
#define LEVEL_6	0x0199	// LEVEL_7 - 3dB 2.00V
#define LEVEL_5	0x0123	// LEVEL_6 - 3dB 1.42V
#define LEVEL_4	0x00CD	// LEVEL_5 - 3dB 1.00V
#define LEVEL_3	0x0091	// LEVEL_4 - 3dB 0.71V
#define LEVEL_2	0x0066	// LEVEL_3 - 3dB 0.50V
#define LEVEL_1	0x004A	// LEVEL_2 - 3dB 0.36V
#define LEVEL_OFFSET	0x0066	// 500mv 
#define LEVEL_RELAY_ON	LEVEL_6
#define FREQ_0	0x0080	// set FREQ ROW0
#define FREQ_1	0x0100	// set FREQ ROW1
#define FREQ_2	0x0180	// set FREQ ROW2
#define FREQ_3	0x0200	// set FREQ ROW3
#define FREQ_4	0x0280	// set FREQ ROW4
#define FREQ_5	0x0300	// set FREQ ROW5
#define FREQ_6	0x0380	// set FREQ ROW6
#define T1_TIME 0xFF83  // 1ms
#define SET_R_COUNT   1000;   // 1s
#define SET_M_COUNT   100;   // 100ms
//
#byte WPUA = 0x20C
#byte WPUB = 0x20D
#byte WPUC = 0x20E
#byte WPUE = 0x210
#byte OPTION_REG = 0x095
#byte ANSELA = 0x18C
#byte ANSELB = 0x18D
#byte ANSELC = 0x18E
//
// variables
int16 val_msgeq7_out[8];
int16 val_msgeq7_sum;
int16 val_freq_pot;
int16 val_disp;
int8 pos_disp = 0;
int8 pos_freq;
int16 r_count1 = 0;
int16 m_count1 = 0;
//
// prototyping
#separate
void initializing();
#separate
void get_msgeq7();
#separate
void get_freq();
#separate
void disp_sound(int8 pos);
#separate
void disp_set_freq();
//
// main function
void main()
{
    initializing();
    // main loop
        
    m_count1 = SET_M_COUNT;
    while(1)
    {
		if(m_count1 == 0)
		{
    		get_msgeq7();
            m_count1 = SET_M_COUNT;
		}
   		get_freq();

        if(input(PIN_C3) == 1)	// SET FREQ SW OFF
        {
			disp_sound(pos_disp);
			if(pos_freq != 7 && val_msgeq7_out[pos_freq] > LEVEL_RELAY_ON)
			{
				RELAY_ON;
				r_count1 = SET_R_COUNT;
			}
			if(r_count1 == 0)
			{
				RELAY_OFF;
			}
		}
		else	// SET FREQ SW ON
		{
			RELAY_OFF;
			disp_set_freq();
		}
		pos_disp++;
        if(pos_disp > 7) pos_disp = 0;
		delay_us(100);
		
	}
}
//
#separate
void initializing()
{
	setup_oscillator(OSC_4MHZ);
	//
	set_tris_a(Amode);   //
	set_tris_b(Bmode);   //
	set_tris_c(Cmode);   //
	port_a_pullups(AmodePU);
	port_b_pullups(BmodePU);
	port_c_pullups(CmodePU);
    OPTION_REG = OPTION_REG & 0x7F;	// set 0 to WPUEN
    ANSELA = 0x37;	//  A0-A2,A4-A5 analog
	ANSELB = 0x00;	// all digital
	ANSELC = 0x00;	// all digital
	
	setup_adc(ADC_CLOCK_DIV_8);   // 2usec
	setup_adc_ports(sAN0 | sAN2 | sAN3 | VSS_VDD);    // PIN_A0,A2,A3
	//
	setup_opamp1(OPAMP_ENABLED | OPAMP_HIGH_GBWP_MODE | OPAMP_I_TO_OPAIN | OPAMP_NI_TO_OPAIN);
	// timer 1 intialize for hysteresis count down
	setup_timer_1(T1_INTERNAL|T1_DIV_BY_8); //8us
	set_timer1(T1_TIME);              // timer 1 setting
	enable_interrupts(INT_TIMER1);   // timer 1 enable
	enable_interrupts(GLOBAL);       // global enable
    //
    RELAY_OFF;
    MSGEQ7_RESET_OFF;
    MSGEQ7_STROBE_OFF;
    LED_COL1_OFF;
    LED_COL2_OFF;
    LED_COL3_OFF;
    LED_COL4_OFF;
    LED_COL5_OFF;
    LED_COL6_OFF;
    LED_COL7_OFF;
    LED_COL8_OFF;
    LED_ROW_C0_OFF;
    LED_ROW_C1_OFF;
    LED_ROW_C2_OFF;
    //
    pos_disp = 0;
    pos_freq = 7;
    
    delay_ms(100);
}
// get sond from microphone
#separate
void get_msgeq7()
{
	int8 i;
    int16 temp_val;
	
	set_adc_channel(0);   // MSGEQ7 out AN0
	delay_us(100);	// wait Tacq(typ 5us)

    MSGEQ7_STROBE_OFF;
    MSGEQ7_RESET_OFF;
	delay_us(10);
    MSGEQ7_STROBE_ON;
    MSGEQ7_RESET_ON;
	delay_us(100);	// ts 18us min
    MSGEQ7_STROBE_OFF;
	delay_us(100);	// tr 100ns min
    MSGEQ7_RESET_OFF;

	for(i=0;i<7;i++)
	{
        MSGEQ7_STROBE_ON;
        delay_us(100);	//trs 72us min
	    MSGEQ7_STROBE_OFF;
		delay_us(50);	// to 36us min
		temp_val = read_adc();
        if(temp_val > LEVEL_OFFSET) val_msgeq7_out[i] = temp_val - LEVEL_OFFSET;
        else val_msgeq7_out[i] = 0;
		delay_us(50);
	}

    val_msgeq7_out[7] = 0;
}
// get freq potentiometer
#separate
void get_freq()
{
	
	set_adc_channel(3);   // Mic level AN3
	delay_us(100);	// wait Tacq(typ 5us)
	val_freq_pot = read_adc();

	if(val_freq_pot < FREQ_0) pos_freq = 0;
	else if(val_freq_pot < FREQ_1)  pos_freq = 1;
	else if(val_freq_pot < FREQ_2)  pos_freq = 2;
	else if(val_freq_pot < FREQ_3)  pos_freq = 3;
	else if(val_freq_pot < FREQ_4)  pos_freq = 4;
	else if(val_freq_pot < FREQ_5)  pos_freq = 5;
	else if(val_freq_pot < FREQ_6)  pos_freq = 6;
	else pos_freq = 7;

}
//
#separate
void disp_sound(int8 pos)
{
    int8 i;
	
	LED_COL8_OFF;
	LED_COL7_OFF;
	LED_COL6_OFF;
	LED_COL5_OFF;
	LED_COL4_OFF;
	LED_COL3_OFF;
	LED_COL2_OFF;
	LED_COL1_OFF;
	
	switch(pos)
	{
		case 0:
			LED_ROW_C0_OFF;
			LED_ROW_C1_OFF;
			LED_ROW_C2_OFF;
			break;
		case 1:
			LED_ROW_C0_ON;
			LED_ROW_C1_OFF;
			LED_ROW_C2_OFF;
			break;
		case 2:
			LED_ROW_C0_OFF;
			LED_ROW_C1_ON;
			LED_ROW_C2_OFF;
			break;
		case 3:
			LED_ROW_C0_ON;
			LED_ROW_C1_ON;
			LED_ROW_C2_OFF;
			break;
		case 4:
			LED_ROW_C0_OFF;
			LED_ROW_C1_OFF;
			LED_ROW_C2_ON;
			break;
		case 5:
			LED_ROW_C0_ON;
			LED_ROW_C1_OFF;
			LED_ROW_C2_ON;
			break;
		case 6:
			LED_ROW_C0_OFF;
			LED_ROW_C1_ON;
			LED_ROW_C2_ON;
			break;
		case 7:
			LED_ROW_C0_ON;
			LED_ROW_C1_ON;
			LED_ROW_C2_ON;
			break;
		default:
			LED_ROW_C0_OFF;
			LED_ROW_C1_OFF;
			LED_ROW_C2_OFF;
			break;
			
	}
	
	if(pos < 7)
	{
   		val_disp = val_msgeq7_out[pos];
        if(val_disp > LEVEL_8) LED_COL8_ON;
        if(val_disp > LEVEL_7) LED_COL7_ON;
 		if(val_disp > LEVEL_6) LED_COL6_ON;
       	if(val_disp > LEVEL_5) LED_COL5_ON;
        if(val_disp > LEVEL_4) LED_COL4_ON;
       	if(val_disp > LEVEL_3) LED_COL3_ON;
        if(val_disp > LEVEL_2) LED_COL2_ON;
       	if(val_disp > LEVEL_1) LED_COL1_ON;
	}
	else
	{
        val_msgeq7_sum = 0;
        for(i=0;i<7;i++)
        {
            val_msgeq7_sum += val_msgeq7_out[i];
        }
        val_disp = val_msgeq7_sum/2;
	    if(val_disp > LEVEL_8) LED_COL8_ON;
		if(val_disp > LEVEL_7) LED_COL7_ON;
		if(val_disp > LEVEL_6) LED_COL6_ON;
		if(val_disp > LEVEL_5) LED_COL5_ON;
		if(val_disp > LEVEL_4) LED_COL4_ON;
		if(val_disp > LEVEL_3) LED_COL3_ON;
		if(val_disp > LEVEL_2) LED_COL2_ON;
		if(val_disp > LEVEL_1) LED_COL1_ON;
	}
	
}
//
#separate
void disp_set_freq()
{
	
    LED_COL8_OFF;
    LED_COL7_OFF;
    LED_COL6_OFF;
    LED_COL5_OFF;
    LED_COL4_OFF;
    LED_COL3_OFF;
    LED_COL2_OFF;
    LED_COL1_OFF;

	switch(pos_freq)
	{
		case 0:
			LED_ROW_C0_OFF;
			LED_ROW_C1_OFF;
			LED_ROW_C2_OFF;
			break;
		case 1:
			LED_ROW_C0_ON;
			LED_ROW_C1_OFF;
			LED_ROW_C2_OFF;
			break;
		case 2:
			LED_ROW_C0_OFF;
			LED_ROW_C1_ON;
			LED_ROW_C2_OFF;
			break;
		case 3:
			LED_ROW_C0_ON;
			LED_ROW_C1_ON;
			LED_ROW_C2_OFF;
			break;
		case 4:
			LED_ROW_C0_OFF;
			LED_ROW_C1_OFF;
			LED_ROW_C2_ON;
			break;
		case 5:
			LED_ROW_C0_ON;
			LED_ROW_C1_OFF;
			LED_ROW_C2_ON;
			break;
		case 6:
			LED_ROW_C0_OFF;
			LED_ROW_C1_ON;
			LED_ROW_C2_ON;
			break;
		default:
			LED_ROW_C0_ON;
			LED_ROW_C1_ON;
			LED_ROW_C2_ON;
            return; 
			break;
	}
	
    LED_COL8_ON;
	LED_COL7_ON;
	LED_COL6_ON;
	LED_COL5_ON;
	LED_COL4_ON;
	LED_COL3_ON;
	LED_COL2_ON;
	LED_COL1_ON;
}
// timer 1 inturruption
#int_timer1
void isr1(void)
{
   set_timer1(T1_TIME);             // timer 1
   if(r_count1 > 1) r_count1 = r_count1 - 1;
   else r_count1 = 0;
   if(m_count1 > 1) m_count1 = m_count1 - 1;
   else m_count1 = 0;

}
//

